/* List of defines generated by configure. Included with preprocessor flag,
 * -include, to avoid long list of -D defines on the compile command-line.
 * Do not edit.
 */

#ifndef _MOZILLA_CONFIG_H_
#define _MOZILLA_CONFIG_H_

#define ACCESSIBILITY 1
#define ATK_MAJOR_VERSION 1
#define ATK_MINOR_VERSION 12
#define ATK_REV_VERSION 2
#define ATTRIBUTE_ALIGNED_MAX 64
#define D_INO d_ino
#define FUNCPROTO 15
#define HAVE_CURL_CURL_H 1
#define HAVE_DIRENT_H 1
#define HAVE_DLADDR 1
#define HAVE_FCHMOD 1
#define HAVE_FLOCKFILE 1
#define HAVE_FT_BITMAP_SIZE_Y_PPEM 1
#define HAVE_FT_GLYPHSLOT_EMBOLDEN 1
#define HAVE_FT_LOAD_SFNT_TABLE 1
#define HAVE_FT_SELECT_SIZE 1
#define HAVE_GETOPT_H 1
#define HAVE_GNU_GET_LIBC_VERSION 1
#define HAVE_GNU_LIBC_VERSION_H 1
#define HAVE_I18N_LC_MESSAGES 1
#define HAVE_INT16_T 1
#define HAVE_INT32_T 1
#define HAVE_INT64_T 1
#define HAVE_INTTYPES_H 1
#define HAVE_ISATTY 1
#define HAVE_LANGINFO_CODESET 1
#define HAVE_LCHOWN 1
#define HAVE_LIBASOUND 1
#define HAVE_LIBDL 1
#define HAVE_LIBM 1
#define HAVE_LOCALTIME_R 1
#define HAVE_LSTAT64 1
#define HAVE_MALLOC_H 1
#define HAVE_MEMMOVE 1
#define HAVE_MEMORY_H 1
#define HAVE_NL_TYPES_H 1
#define HAVE_RANDOM 1
#define HAVE_RES_NINIT 1
#define HAVE_RINT 1
#define HAVE_SETBUF 1
#define HAVE_SIGINFO_T 1
#define HAVE_SNPRINTF 1
#define HAVE_STAT64 1
#define HAVE_STDINT_H 1
#define HAVE_STRERROR 1
#define HAVE_STRTOK_R 1
#define HAVE_ST_BLKSIZE 1
#define HAVE_SYS_BITYPES_H 1
#define HAVE_SYS_CDEFS_H 1
#define HAVE_SYS_STATFS_H 1
#define HAVE_SYS_STATVFS_H 1
#define HAVE_TRUNCATE64 1
#define HAVE_UINT 1
#define HAVE_UINT64_T 1
#define HAVE_UNAME_DOMAINNAME_FIELD 1
#define HAVE_UNISTD_H 1
#define HAVE_VA_COPY 1
#define HAVE_VISIBILITY_ATTRIBUTE 1
#define HAVE_VISIBILITY_HIDDEN_ATTRIBUTE 1
#define HAVE_X11_XKBLIB_H 1
#define HAVE_XSHM 1
#define HAVE__UNWIND_BACKTRACE 1
#define HAVE___CXA_DEMANGLE 1
#define IBMBIDI 1
#define MOZILLA_VERSION "1.9.2b1pre"
#define MOZILLA_VERSION_U 1.9.2b1pre
#define MOZ_ACCESSIBILITY_ATK 1
#define MOZ_BUILD_APP xulrunner
#define MOZ_CRASHREPORTER 1
#define MOZ_CRASHREPORTER_ENABLE_PERCENT 100
#define MOZ_DEFAULT_TOOLKIT "cairo-gtk2"
#define MOZ_DISTRIBUTION_ID "org.mozilla"
#define MOZ_DLL_SUFFIX ".so"
#define MOZ_EMBEDDING_LEVEL_BASIC 1
#define MOZ_EMBEDDING_LEVEL_DEFAULT 1
#define MOZ_EMBEDDING_LEVEL_MINIMAL 1
#define MOZ_ENABLE_CANVAS 1
#define MOZ_ENABLE_DBUS 1
#define MOZ_ENABLE_GCONF 1
#define MOZ_ENABLE_GNOMEUI 1
#define MOZ_ENABLE_GNOMEVFS 1
#define MOZ_ENABLE_LIBNOTIFY 1
#define MOZ_ENABLE_LIBXUL 1
#define MOZ_ENABLE_OLD_ABI_COMPAT_WRAPPERS 1
#define MOZ_ENABLE_XREMOTE 1
#define MOZ_FEEDS 1
#define MOZ_JSLOADER 1
#define MOZ_LOGGING 1
#define MOZ_MATHML 1
#define MOZ_MEDIA 1
#define MOZ_MEMORY 1
#define MOZ_MEMORY_LINUX 1
#define MOZ_MEMORY_SIZEOF_PTR_2POW 2
#define MOZ_MORKREADER 1
#define MOZ_NO_XPCOM_OBSOLETE 1
#define MOZ_OGG 1
#define MOZ_PANGO 1
#define MOZ_PLACES 1
#define MOZ_PROFILELOCKING 1
#define MOZ_RDF 1
#define MOZ_SMIL 1
#define MOZ_STORAGE 1
#define MOZ_SVG 1
#define MOZ_SYDNEYAUDIO 1
#define MOZ_TREE_CAIRO 1
#define MOZ_UPDATE_CHANNEL default
#define MOZ_USER_DIR ".mozilla"
#define MOZ_VIEW_SOURCE 1
#define MOZ_WAVE 1
#define MOZ_WIDGET_GTK2 1
#define MOZ_X11 1
#define MOZ_XPINSTALL 1
#define MOZ_XTF 1
#define MOZ_XUL 1
#define MOZ_XULRUNNER 1
#define NS_PRINTING 1
#define NS_PRINT_PREVIEW 1
#define SIZEOF_INT_P 4
#define STDC_HEADERS 1
#define UNIX_ASYNC_DNS 1
#define VA_COPY va_copy
#define XP_UNIX 1
#define _REENTRANT 1

#endif /* _MOZILLA_CONFIG_H_ */

